﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StorageContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDatabaseImplement.Models;

namespace SecuritySystemDatabaseImplement.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            if (string.IsNullOrEmpty(model.MessageId))
            {
                return null;
            }
            using var context = new SecuritySystemDatabase();
            return context.MessageInfos
          .FirstOrDefault(x => x.MessageId == model.MessageId)
            ?.GetViewModel;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            if (!model.ClientId.HasValue)
            {
                return new();
            }
            using var context = new SecuritySystemDatabase();
            return context.MessageInfos
            .Where(x => x.ClientId == model.ClientId)
           .Select(x => x.GetViewModel)
           .ToList();
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            using var context = new SecuritySystemDatabase();
            return context.MessageInfos
            .Select(x => x.GetViewModel)
           .ToList();
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {
            var newMessage = MessageInfo.Create(model);
            if (newMessage == null)
            {
                return null;
            }
            using var context = new SecuritySystemDatabase();
            context.MessageInfos.Add(newMessage);
            context.SaveChanges();
            return newMessage.GetViewModel;
        }
    }
}
