﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Enums;
using SecuritySystemDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.Reflection;
using System.Runtime.Serialization;

namespace SecuritySystemDatabaseImplement.Models
{
    [DataContract]
    public class Order : IOrderModel
    {
        [DataMember]
        public int Id { get; set; }
        [Required]
        [DataMember]
        public int SecureId { get; set; }
        [Required]
        [DataMember]
        public int ClientId { get; set; }
        public int? ImplementerId { get; set; }
        [Required]
        [DataMember]
        public int Count { get; set; }
        [Required]
        [DataMember]
        public double Sum { get; set; }
        [Required]
        [DataMember]
        public OrderStatus Status { get; set; }
        [Required]
        [DataMember]
        public DateTime DateCreate { get; set; }
        [DataMember]

        public DateTime? DateImplement { get; set; }
        [DataMember]

        public virtual Secure Secure { get; set; }
        public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                SecureId = model.SecureId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SecureId = SecureId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            SecureName = Secure.SecureName,
            ClientFIO = Client.ClientFIO,
            ImplementerFIO = Implementer == null ? null : Implementer.ImplementerFIO
        };
    }
}
