﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemDataModels.Models;
using System.Runtime.Serialization;
using System.Xml.Linq;
using SecuritySystemContracts.ViewModels;

namespace SecuritySystemFileImplement.Models
{
    [DataContract]
    public class Implementer : IImplementerModel
    {
        [DataMember]
        public int Id { get; private set; }
        [DataMember]
        public string ImplementerFIO { get; private set; } = string.Empty;
        [DataMember]
        public string Password { get; private set; } = string.Empty;
        [DataMember]
        public int WorkExperience { get; private set; }
        [DataMember]
        public int Qualification { get; private set; }

        public static Implementer? Create(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Implementer()
            {
                Id = model.Id,
                ImplementerFIO = model.ImplementerFIO,
                Password = model.Password,
                WorkExperience = model.WorkExperience,
                Qualification = model.Qualification,
            };
        }
        public static Implementer? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Implementer()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ImplementerFIO = element.Element("ImplementerFIO")!.Value,
                Password = element.Element("Password")!.Value,
                WorkExperience = Convert.ToInt32(element.Attribute("WorkExperience")!.Value),
                Qualification = Convert.ToInt32(element.Attribute("Qualification")!.Value),
            };
        }
        public void Update(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ImplementerFIO = model.ImplementerFIO;
            Password = model.Password;
            WorkExperience = model.WorkExperience;
            Qualification = model.Qualification;
        }
        public ImplementerViewModel GetViewModel => new()
        {
            Id = Id,
            ImplementerFIO = ImplementerFIO,
            Password = Password,
            WorkExperience = WorkExperience,
            Qualification = Qualification
        };

        public XElement GetXElement => new("Implementer",
            new XAttribute("Id", Id),
            new XElement("ImplementerFIO", ImplementerFIO),
            new XElement("Password", Password),
            new XElement("WorkExperience", WorkExperience),
            new XElement("Qualification", Qualification));
    }
}
