﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemDataModels.Enums;
using SecuritySystemDataModels.Models;
using System.Runtime.Serialization;
using System.Xml.Linq;
using SecuritySystemContracts.ViewModels;

namespace SecuritySystemFileImplement.Models
{
    [DataContract]
    public class Order : IOrderModel
    {
        [DataMember]
        public int SecureId { get; private set; }
        [DataMember]
        public int ClientId { get; private set; }
        [DataMember]
        public int? ImplementerId { get; private set; }
        [DataMember]
        public int Count { get; private set; }
        [DataMember]
        public double Sum { get; private set; }
        [DataMember]
        public OrderStatus Status { get; private set; }
        [DataMember]
        public DateTime DateCreate { get; private set; }
        [DataMember]
        public DateTime? DateImplement { get; private set; }
        [DataMember]
        public int Id { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                SecureId = model.SecureId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                SecureId = Convert.ToInt32(element.Element("SecureId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null),
                DateImplement = Convert.ToDateTime(element.Element("DateCreate")!.Value)
            };
            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);
            order.DateImplement = dateImpl;
            if (!Enum.TryParse(element.Element("Status")!.Value, out OrderStatus status))
            {
                return null;
            }
            order.Status = status;
            return order;
        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            SecureId = model.SecureId;
            ClientId = model.ClientId;
            ImplementerId = model.ImplementerId;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SecureId = SecureId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
        public XElement GetXElement => new("Order",
        new XAttribute("Id", Id),
        new XElement("SecureId", SecureId.ToString()),
        new XElement("ClientId", ClientId.ToString()),
        new XElement("ImplementerId", ClientId.ToString()),
        new XElement("Count", Count.ToString()),
        new XElement("Sum", Sum.ToString()),
        new XElement("Status", Status.ToString()),
        new XElement("DateCreate", DateCreate.ToString()),
        new XElement("DateImplement", DateImplement.ToString()));


    }
}
